package java.time.temporal;

import java.time.chrono.Chronology;
import java.time.chrono.ChronoLocalDate;
import java.time.format.ResolverStyle;
import java.util.Map;
import java.time.DateTimeException;

public final class JulianFields
{
    private static final long JULIAN_DAY_OFFSET = 2440588L;
    public static final TemporalField JULIAN_DAY;
    public static final TemporalField MODIFIED_JULIAN_DAY;
    public static final TemporalField RATA_DIE;
    
    private JulianFields() {
        throw new AssertionError("Not instantiable");
    }
    
    static {
        JULIAN_DAY = JulianFields.Field.JULIAN_DAY;
        MODIFIED_JULIAN_DAY = JulianFields.Field.MODIFIED_JULIAN_DAY;
        RATA_DIE = JulianFields.Field.RATA_DIE;
    }
    
    private enum Field implements TemporalField
    {
        JULIAN_DAY("JulianDay", ChronoUnit.DAYS, ChronoUnit.FOREVER, 2440588L), 
        MODIFIED_JULIAN_DAY("ModifiedJulianDay", ChronoUnit.DAYS, ChronoUnit.FOREVER, 40587L), 
        RATA_DIE("RataDie", ChronoUnit.DAYS, ChronoUnit.FOREVER, 719163L);
        
        private static final long serialVersionUID = -7501623920830201812L;
        private final transient String name;
        private final transient TemporalUnit baseUnit;
        private final transient TemporalUnit rangeUnit;
        private final transient ValueRange range;
        private final transient long offset;
        
        private Field(String name2, TemporalUnit baseUnit, TemporalUnit rangeUnit, long offset) {
            this.name = name2;
            this.baseUnit = baseUnit;
            this.rangeUnit = rangeUnit;
            this.range = ValueRange.of(-365243219162L + offset, 365241780471L + offset);
            this.offset = offset;
        }
        
        @Override
        public TemporalUnit getBaseUnit() {
            return this.baseUnit;
        }
        
        @Override
        public TemporalUnit getRangeUnit() {
            return this.rangeUnit;
        }
        
        @Override
        public boolean isDateBased() {
            return true;
        }
        
        @Override
        public boolean isTimeBased() {
            return false;
        }
        
        @Override
        public ValueRange range() {
            return this.range;
        }
        
        @Override
        public boolean isSupportedBy(TemporalAccessor temporalAccessor) {
            return temporalAccessor.isSupported(ChronoField.EPOCH_DAY);
        }
        
        @Override
        public ValueRange rangeRefinedBy(TemporalAccessor temporalAccessor) {
            if (!this.isSupportedBy(temporalAccessor)) {
                throw new DateTimeException("Unsupported field: " + this);
            }
            return this.range();
        }
        
        @Override
        public long getFrom(TemporalAccessor temporalAccessor) {
            return temporalAccessor.getLong(ChronoField.EPOCH_DAY) + this.offset;
        }
        
        @Override
        public <R extends Temporal> R adjustInto(R r, long n) {
            if (!this.range().isValidValue(n)) {
                throw new DateTimeException("Invalid value: " + this.name + " " + n);
            }
            return r.with(ChronoField.EPOCH_DAY, Math.subtractExact(n, this.offset));
        }
        
        @Override
        public ChronoLocalDate resolve(Map<TemporalField, Long> map, TemporalAccessor temporal, ResolverStyle resolverStyle) {
            long longValue = (long)map.remove(this);
            Chronology from = Chronology.from(temporal);
            if (resolverStyle == ResolverStyle.LENIENT) {
                return from.dateEpochDay(Math.subtractExact(longValue, this.offset));
            }
            this.range().checkValidValue(longValue, this);
            return from.dateEpochDay(longValue - this.offset);
        }
        
        @Override
        public String toString() {
            return this.name;
        }
    }
}
